# author: C.Cicconetti <claudio.cicconetti@iet.unipi.it>
#
# creates a grid of wireless 802.11 nodes at distance 50m equal to the
# transmission range at which packets experience 0.95 corruption probability
# on average (using the shadowing path model)
#
# each node has an FTP flow with a randomly (uniform distribution) chosen node
#



# system parameters
set opt(radius)    2      ;# grid radius, in number of nodes
set opt(lambda)    50     ;# distance, in meters
set opt(run)       0      ;# replic ID
set opt(duration)  100.0 ;# run duration, in seconds
set opt(warm)      50.0  ;# run duration, in seconds
set opt(out)       "out"  ;# FIFO out
set opt(trc)       ""

proc getopt {argc argv} {
   global opt

   for {set i 0} {$i < $argc} {incr i} {
      set arg [lindex $argv $i]
      if {[string range $arg 0 0] != "-"} continue

      set name [string range $arg 1 end]
      set opt($name) [lindex $argv [expr $i+1]]
   }
}


proc finish {} {
   global ns opt      ;# input

	# flush traces
	$ns flush-trace

	# close trace files
	close $opt(trace)

	if { $opt(trc) == "" } {
		$ns stat print
	}

   exit 0
}

proc init {} {
   global opt defaultRNG ns

	# create the simulator instance
   set ns [new Simulator]  ;# create a new simulator instance
	$defaultRNG seed 1

	# initialize statistics collection
	$ns run-identifier $opt(run)
	$ns at $opt(duration) "finish"

   # open trace files
	if { $opt(trc) == "" } {
		set opt(trace) [open "/dev/null" w]
		$ns stat file "$opt(out)"
		$ns at $opt(warm) "$ns stat on"
	} else {
		set opt(trace) [open $opt(trc) w]
	}

   $ns use-newtrace
	$ns trace-all $opt(trace)
}

proc create_topology {} {
   global ns opt topo sta defaultRNG

	# set PHY parameters (IEEE802.11b)
	Mac/802_11 set SlotTime_          0.000020        ;# 20us
	Mac/802_11 set SIFS_              0.000010        ;# 10us
	Mac/802_11 set PreambleLength_    144             ;# 144 bits
	Mac/802_11 set PLCPHeaderLength_  48              ;# 48 bits
	Mac/802_11 set PLCPDataRate_      1.0e6           ;# 1Mbps
	Mac/802_11 set dataRate_          2.0e6           ;# 11Mbps
	Mac/802_11 set basicRate_         1.0e6           ;# 1Mbps

	# set MAC parameters
	Mac/802_11 set CWMin_             31      ;# minimum contention window
	Mac/802_11 set CWMax_             1023    ;# maximum contention window
	Mac/802_11 set RTSThreshold_      1000    ;# RTS threshold
	Mac/802_11 set ShortRetryLimit_   7       ;# short retry limit
	Mac/802_11 set LongRetryLimit_    4       ;# long retry limit

	# set PHY parameters
	Phy/WirelessPhy set RXThresh_ 1.69063e-08 ;# 50m range

   # compute the area length, in meters
	set length [expr $opt(lambda) * ( 2 + $opt(radius))]

	# compute the number of nodes
	set opt(n) [expr $opt(radius) * $opt(radius)]

   set topo [new Topography]      ;# Create topography
   create-god [expr $opt(n)]      ;# create the General Operations Director
   $topo load_flatgrid $length $length        ;# load a flat grid
   set chan_1 [new Channel/WirelessChannel]   ;# create the wireless channel

	;# set the node configuration
   $ns node-config \
	   -adhocRouting AODV \
      -llType LL \
      -macType Mac/802_11 \
      -ifqType Queue/DropTail/PriQueue \
      -ifqLen 50 \
      -antType Antenna/OmniAntenna \
      -propType Propagation/Shadowing \
      -phyType Phy/WirelessPhy \
      -topoInstance $topo \
      -agentTrace ON \
      -routerTrace ON \
      -macTrace  ON \
      -movementTrace  ON \
      -channel $chan_1

	# turn on the End of Transmission trace
	Simulator set EotTrace_ ON

   # creating mobile nodes
   for {set i 0} {$i < $opt(radius)} {incr i} {
		for {set j 0} {$j < $opt(radius)} {incr j} {
			set ndx [expr $i * $opt(radius) + $j]
			set sta($ndx) [$ns node]      ;# create new node
			$sta($ndx) random-motion 0    ;# disable random motion

			$sta($ndx) set X_ [expr ( 1 + $j ) * $opt(lambda)]
			$sta($ndx) set Y_ [expr ( 1 + $i ) * $opt(lambda)]
			$sta($ndx) set Z_ 0
			$ns initial_node_pos $sta($ndx) 20   ;# for NAM

			puts "node $ndx ([$sta($ndx) set X_], [$sta($ndx) set Y_]) created"
		}
   }
}

proc create_connections {} {
	global sta ns opt defaultRNG

	set rng_dst   [new RNG]
	set rng_start [new RNG]

	for { set i 0 } { $i < $opt(n) } { incr i } {
		if { $i == [expr $opt(n)/2] } {
			continue
		}

		set start_time [$rng_start uniform 0 [expr $opt(warm)/2]]
		set end_time   $opt(duration)
		set dst        [expr $opt(n)/2]

		set application($i) [new Application/FTP]
		$ns at $start_time "$application($i) start"
		$ns at $end_time   "$application($i) stop"

		puts "$i -> $dst, from $start_time to $end_time"

		# create agents
		set agtsrc($i) [new Agent/TCP]
		set agtdst($i) [new Agent/TCPSink]
		$agtsrc($i) set class_ [expr $i]
		$agtsrc($i) set packetSize_ 1500

		# connect agents
		$ns attach-agent $sta($i) $agtsrc($i)
		$ns attach-agent $sta($dst) $agtdst($i)
		$ns connect $agtsrc($i) $agtdst($i)
		$application($i) attach-agent $agtsrc($i)

		# connect e2e tagger and monitor
		set tagger($i)  [new e2et]
		set monitor($i) [new e2em]
		$agtsrc($i) attach-e2et $tagger($i)
		$agtdst($i) attach-e2em $monitor($i)
		$monitor($i) start-log
	}
}

# main body starts here
getopt $argc $argv

init
create_topology
create_connections

$ns run
